package main

import (
	"encoding/json"
	"log"
	"net/http"
	"os"

	"github.com/gorilla/mux"
)

func main() {
	log := log.New(os.Stderr, "", log.LstdFlags)
	addr := ":8080"
	log.Print("Listening on ", addr)
	err := http.ListenAndServe(addr, NewRouter())
	if err != nil {
		log.Print(err)
		os.Exit(1)
	}
}

func NewRouter() http.Handler {
	router := mux.NewRouter()
	books := &books{}
	router.Handle("/books", books).Methods("GET")
	return logIncoming(router)
}

func logIncoming(next http.Handler) http.HandlerFunc {
	log := log.New(os.Stderr, "", log.LstdFlags)
	return func(w http.ResponseWriter, r *http.Request) {
		log.Printf("%s %s", r.Method, r.URL.Path)
		next.ServeHTTP(w, r)
	}
}

type books struct{}

func (h *books) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	w.Header().Set("Content-Type", "application/json")
	model := struct {
		Message string `json:"message"`
	}{
		"No books available!",
	}
	json.NewEncoder(w).Encode(model)
}
